from pyrediseasyio.abstract_reader_writer import AbstractReaderWriter
from pyrediseasyio import SingleIO


class StringIO(SingleIO):
    def __init__(self, name: str, addr: str, default: str = '', units: str = None, reader: AbstractReaderWriter = None):
        super().__init__(name, addr, default, units, reader)

    @staticmethod
    def _convert_type(value):
        return str(value)


from pyrediseasyio.io.io_group import IOGroup
from pyrediseasyio.html.html_io import HTMLIO
from typing import List, Callable
from dominate.tags import div, table
import json


class HMTLIOGroup:

    def __init__(self, io_group: IOGroup,
                 html_id_header: str,
                 namespace: str = 'pyredeio'):

        self.html_id_header = html_id_header
        self._io_group = io_group
        self.namespace = namespace

    @property
    def html_id(self):
        return f'{self.html_id_header}_io_container'

    def html(self, by_names: List = None, by_type: List = None,
             by_lambda_each: Callable = None, by_lambda_results: Callable = None,
             html_classes: List = None):

        attrs = self._io_group.get_attributes(by_names, by_type, by_lambda_each, by_lambda_results)
        cls = f'{self.namespace}_io_container'
        classes = html_classes.append(cls) if html_classes else [cls]
        classes = ' '.join(classes)

        with div(cls=classes, id=self.html_id) as container:
            for attr in attrs:
                HTMLIO(attr, self.html_id_header, self.namespace).html()
        return container

    def html_table(self, by_names: List = None, by_type: List = None,
                   by_lambda_each: Callable = None, by_lambda_results: Callable = None,
                   html_classes: List = None, show_units: bool=True):

        attrs = self._io_group.get_attributes(by_names, by_type, by_lambda_each, by_lambda_results)
        cls = f'{self.namespace}_io_container'
        classes = html_classes.append(cls) if html_classes else [cls]
        classes = ' '.join(classes)

        with table(cls=classes, id=self.html_id) as container:
            for attr in attrs:
                HTMLIO(attr, self.html_id_header, self.namespace).html_row(show_units)
        return container

    def dumps(self, by_names: List = None, by_type: List = None,
              by_lambda_each: Callable = None, by_lambda_results: Callable = None):

        attrs = self._io_group.get_attributes(by_names, by_type, by_lambda_each, by_lambda_results)

        def f(a):
            return dict(id=HTMLIO.html_id_for(self.html_id_header, a), name=a.name, value=a.value, units=a.units)
        results = [f(a) for a in attrs]

        return json.dumps(results)





from pyrediseasyio.io.base import SingleIO
from dominate.tags import div, tr, td



class HTMLIO:

    def __init__(self, io: SingleIO, html_id_header: str, namespace: str = 'pyredeio'):
        self.io = io
        self.namespace = namespace
        self.html_id_header = html_id_header

    @staticmethod
    def html_id_for(header: str, attr: SingleIO):
        return f'{header}_{attr.addr}_io'

    @property
    def html_id(self):
        return self.html_id_for(self.html_id_header, self.io)

    @property
    def html_class(self):
        return f'{self.namespace}_io'

    def html(self, show_units: bool = True):
        io = self.io
        name, addr, val, units = io.name, io.addr, io.value, io.units
        with div(cls=self.html_class, id=self.html_id) as container:
            div(name, cls=f'{self.html_class}_name')
            div(val, cls=f'{self.html_class}_value', id=f'{self.html_id}_value', onchange='OnIOValueChange(event)')
            if show_units:
                units = '' if units is None else units
                div(units, cls=f'{self.html_class}_units')
        return container

    def html_row(self, show_units: bool = True):
        io = self.io
        name, addr, val, units = io.name, io.addr, io.value, io.units
        with tr(cls=self.html_class, id=self.html_id) as container:
            td(name, cls=f'{self.html_class}_name')
            td(val, cls=f'{self.html_class}_value', id=f'{self.html_id}_value', onchange='OnIOValueChange(event)')
            if show_units:
                units = '' if units is None else units
                td(units, cls=f'{self.html_class}_units')
        return container










from pyrediseasyio import SingleIO
from pyrediseasyio.abstract_reader_writer import AbstractReaderWriter
from str2bool import str2bool


class BooleanIO(SingleIO):
    def __init__(self, name: str, addr: str, default: bool = False, units: str = None,
                 reader: AbstractReaderWriter = None):
        super().__init__(name, addr, default, units, reader)

    @staticmethod
    def _convert_type(value):
        if value is None:
            return False
        if isinstance(value, str):
            return str2bool(value)
        return bool(value)

    def __bool__(self):
        return self.value

    @property
    def value(self) -> bool:
        return super().value



import unittest
from pyrediseasyio.io.io_group import IOGroup
from pyrediseasyio import BooleanIO, IntIO, FloatIO
from assertpy import assert_that


class TestGroup(IOGroup):
    length = 5
    Bool1 = BooleanIO("Boolean 1", "Bool1", False)
    Bool2 = BooleanIO("Boolean 2", "Bool2", True)
    Int1 = IntIO("Integer 1", "Int1")
    Int2 = IntIO("Integer 2", "Int2", default=34)
    Float1 = FloatIO("Float 1", "Float1", default=1.2)


test_group = TestGroup()


class TestAttributes(unittest.TestCase):

    def test_get_all_attributes(self):
        attrs = test_group.get_attributes()
        assert_that(len(attrs)).is_equal_to(TestGroup.length)

    def test_limit_attributes_by_names(self):
        attrs = test_group.get_attributes(by_names=['Bool1','Float1'])
        assert_that(len(attrs)).is_equal_to(2)

    def test_limit_attributes_by_type(self):
        attrs = test_group.get_attributes(by_type=[IntIO, FloatIO])
        assert_that(len(attrs)).is_equal_to(3)

    def test_limit_by_io_filter(self):
        attrs = test_group.get_attributes(by_lambda_each=lambda x: x.value == 1.2)
        assert_that(len(attrs)).is_equal_to(1)

    def test_limit_by_io_result(self):
        attrs = test_group.get_attributes(by_lambda_results=lambda x: x[-1:])
        assert_that(len(attrs)).is_equal_to(1)
        assert_that(attrs[0].value).is_equal_to(34)




import unittest
from pyrediseasyio.io.io_group import IOGroup
from pyrediseasyio.html.html_io_group import HMTLIOGroup
from pyrediseasyio import BooleanIO, SingleIO, FloatIO, StringIO
from assertpy import assert_that
from htmldiffer import utils as diff_utils
import json


class TestGroup(IOGroup):
    Bool1 = BooleanIO("Boolean 1", "Bool1", units="On/Off")
    Float1 = FloatIO("Float 1", "Float1", default=1.23, units="furlongs")


test_group = TestGroup()


def html_equals(html1, html2):
    lines1 = [l for l in diff_utils.html2list(html1) if l != '\n ' and l != ' ' and l != '\n']
    lines2 = [l for l in diff_utils.html2list(html2) if l != '\n ' and l != ' ' and l != '\n']
    return lines1 == lines2


class TestHTML(unittest.TestCase):

    def test_boolean_div_conversion(self):
        h = HMTLIOGroup(test_group, "my_id", "my_namespace").html().render()
        expected = '''
        <div class="my_namespace_io_container" id="my_id_io_container">
          <div class="my_namespace_io" id="my_id_Bool1_io">
            <div class="my_namespace_io_name">Boolean 1</div>
            <div class="my_namespace_io_value" id="my_id_Bool1_io_value" onchange="OnIOValueChange(event)">False</div>
            <div class="my_namespace_io_units">On/Off</div>
          </div>
          <div class="my_namespace_io" id="my_id_Float1_io">
            <div class="my_namespace_io_name">Float 1</div>
            <div class="my_namespace_io_value" id="my_id_Float1_io_value" onchange="OnIOValueChange(event)">1.23</div>
            <div class="my_namespace_io_units">furlongs</div>
          </div>
        </div>
        '''
        assert_that(html_equals(h, expected)).is_true()

    def test_boolean_table_conversion(self):
        h = HMTLIOGroup(test_group, "my_id", "my_namespace").html_table().render()
        expected = '''
        <table class="my_namespace_io_container" id="my_id_io_container">
          <tr class="my_namespace_io" id="my_id_Bool1_io">
            <td class="my_namespace_io_name">Boolean 1</td>
            <td class="my_namespace_io_value" id="my_id_Bool1_io_value" onchange="OnIOValueChange(event)">False</td>
            <td class="my_namespace_io_units">On/Off</td>
          </tr>
          <tr class="my_namespace_io" id="my_id_Float1_io">
            <td class="my_namespace_io_name">Float 1</td>
            <td class="my_namespace_io_value" id="my_id_Float1_io_value" onchange="OnIOValueChange(event)">1.23</td>
            <td class="my_namespace_io_units">furlongs</td>
          </tr>
        </table>
        '''
        assert_that(html_equals(h, expected)).is_true()

    def test_json_dumps_basic(self):
        h = HMTLIOGroup(test_group, "my_id", "my_namespace")
        d = h.dumps()
        as_list = json.loads(d)
        assert_that(len(as_list)).is_equal_to(len(test_group))
        assert_that(as_list[0]['name']).is_equal_to('Boolean 1')

    def test_json_dumps_by_name(self):
        h = HMTLIOGroup(test_group, "my_id", "my_namespace")
        d = h.dumps(by_names=['Bool1'])
        as_list = json.loads(d)
        assert_that(len(as_list)).is_equal_to(1)



from pyrediseasyio import SingleIO
from pyrediseasyio.abstract_reader_writer import AbstractReaderWriter


class FloatIO(SingleIO):
    def __init__(self, name: str, addr: str, default: float = 0, units: str = None, reader: AbstractReaderWriter = None):
        super().__init__(name, addr, default, units, reader)

    @staticmethod
    def _convert_type(value):
        return float(value)


